﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Numerics;

namespace RandomBigIntegers
{
    public partial class Form1 : Form
    {
        static Random rand = new Random();

        long sampleSize;
        BigInteger max;
        int[] heights;

        public delegate BigInteger RandomBigIntegerDelegate(BigInteger max);
        RandomBigIntegerDelegate randomMethod;

        public Form1()
        {
            InitializeComponent();
            update();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            if (heights.Max() == 0)
                return;
            double scale = 300.0 / heights.Max();
            for (int i = 0; i < heights.Length; i++)
            {
                int h = (int)(heights[i] * scale);
                e.Graphics.FillRectangle(Brushes.Black, 50 + i, 500 - h, 1, h);
            }
        }

        //returns time in miliseconds required to generate numbers
        private int generateNumbers(long testCount, int width, BigInteger max)
        {
            int begin = System.Environment.TickCount;
            this.heights = new int[width];

            for (long i = 0; i < testCount; i++)
            {
                BigInteger num = this.randomMethod(max);
                int bucket = (int)(num * width / max);
                heights[bucket]++;
            }
            int end = System.Environment.TickCount;
            return end - begin;
        }

        // Generate a number from 1 to max (exclusive)
        private BigInteger randomBigIntegerMod(BigInteger max)
        {
            int length = (int)Math.Ceiling(BigInteger.Log(max, 2) / 31);
            BigInteger random_number = 0;
            for (int i = 0; i < length; i++)
                random_number = (random_number << 31) + rand.Next();
            return random_number % max;
        }
        static Random random = new Random();

        // returns a evenly distributed random BigInteger from 1 to N.
        BigInteger randomBigInteger(BigInteger N)
        {
            BigInteger result = 0;
            do
            {
                int length = (int)Math.Ceiling(BigInteger.Log(N, 2));
                int numBytes = (int)Math.Ceiling(length / 8.0);
                byte[] data = new byte[numBytes];
                random.NextBytes(data);
                result = new BigInteger(data);
            } while (result >= N || result < 0);
            return result;
        }

        private void update()
        {
            if (this.typeMod.Checked)
            {
                this.randomMethod = new RandomBigIntegerDelegate(randomBigIntegerMod);
            }
            else
            {
                this.randomMethod = new RandomBigIntegerDelegate(randomBigInteger);
            }

            sampleSize = (long)Math.Pow(2, this.count.Value);

            this.countLabel.Text = "2^" + this.count.Value + " = " + sampleSize.ToString("n0");

            if (this.maxN.Text == "" || BigInteger.Parse(this.maxN.Text) == 0)
                return;

            max = BigInteger.Parse(this.maxN.Text);
            int milliseconds = generateNumbers(sampleSize, this.Width - 100, max);
            this.duration.Text = "duration: " + milliseconds + " ms";
            this.Refresh();
        }

        private void count_Scroll(object sender, EventArgs e)
        {  
            update();
        }

        private void maxN_TextChanged(object sender, EventArgs e)
        {
            update();
        }

        private void typeMod_CheckedChanged(object sender, EventArgs e)
        {
            update();
        }

        private void typeRepeat_CheckedChanged(object sender, EventArgs e)
        {
            update();
        }
    }
}
